# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Setup Account page Qt UI."""

from PyQt4 import QtGui, QtCore

from ubuntu_sso.qt import current_user_sign_in_page
from ubuntu_sso.qt.tests import (
    BaseTestCasePage,
    FakePageUiStyle,
    FakeWizardButtonStyle,
)


# pylint: disable=W0212
class CurrentUserSignInTestCase(BaseTestCasePage):
    """Test the SetupAccountPage code."""

    ui_class = current_user_sign_in_page.CurrentUserSignInPage
    ui_signals = ('userLoggedIn', 'passwordForgotten', 'userNotValidated')
    ui_backend_signals = ('LoggedIn', 'LoginError', 'UserNotValidated')

    def test_on_user_not_validated(self):
        """Test the navigation flow on user not validated."""
        email = 'email@example'
        password = 'password'
        self.ui.ui.email_edit.setText(email)
        self.ui.ui.password_edit.setText(password)

        def slot(email, password):
            """Fake slot."""
            self.signals_results.append((email, password))
        self.ui.userNotValidated.connect(slot)
        self.ui.on_user_not_validated()
        self.assertEqual(self._overlay_hide_counter, 1)
        self.assertIn((email, password), self.signals_results)

    def test_initialize_page(self):
        """Test the initialization method."""
        wizard = FakeWizardButtonStyle()
        button = FakePageUiStyle()
        self.patch(self.ui, "wizard", lambda: wizard)
        self.patch(self.ui.ui, "sign_in_button", button)
        self.patch(self.ui, "setButtonText", wizard.setButtonText)
        self.ui.initializePage()
        self.assertEqual(wizard.buttons_text[QtGui.QWizard.CancelButton],
            "Cancel")
        self.assertIn(('setButtonLayout',
            (([QtGui.QWizard.BackButton, QtGui.QWizard.Stretch],), {})),
            wizard.called)
        self.assertTrue(button.properties['default'])
        self.assertFalse(button.isEnabled())

    def test_set_translated_strings(self):
        """Test the translated string method."""
        self.ui._set_translated_strings()
        self.assertEqual(self.ui.ui.email_label.text(),
            current_user_sign_in_page.EMAIL_LABEL)
        self.assertEqual(self.ui.ui.password_label.text(),
            current_user_sign_in_page.LOGIN_PASSWORD_LABEL)
        text = current_user_sign_in_page.FORGOTTEN_PASSWORD_LINK_STYLE.format(
            forgotten_text=current_user_sign_in_page.FORGOTTEN_PASSWORD_BUTTON)
        self.assertEqual(self.ui.ui.forgot_password_label.text(),
            text)
        self.assertEqual(self.ui.ui.sign_in_button.text(),
            current_user_sign_in_page.SIGN_IN_BUTTON)

    def test_connect_ui(self):
        """Test the connect ui method."""
        self.ui._connect_ui()
        # We expect 2 values because _connect_ui is called in the init too.
        self.assertEqual(self.ui.ui.forgot_password_label.receivers(
            QtCore.SIGNAL("linkActivated(const QString&)")), 2)
        self.assertEqual(self.ui.ui.email_edit.receivers(
            QtCore.SIGNAL("textChanged(const QString&)")), 2)
        self.assertEqual(self.ui.ui.password_edit.receivers(
            QtCore.SIGNAL("textChanged(const QString&)")), 2)
        self.assertEqual(self.ui.ui.sign_in_button.receivers(
            QtCore.SIGNAL("clicked()")), 2)

    def test_validate_not_valid(self):
        """Test validate method."""
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "sign_in_button", button)
        self.ui.ui.email_edit.setText('not-valid-email')
        self.ui.ui.password_edit.setText('123456')
        self.ui._validate()
        self.assertFalse(button.isEnabled())

    def test_validate_valid(self):
        """Test validate method."""
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "sign_in_button", button)
        self.ui.ui.email_edit.setText('valid@email')
        self.ui.ui.password_edit.setText('123456')
        self.ui._validate()
        self.assertTrue(button.isEnabled())

    def test_login_with_ping(self):
        """Test the login method."""
        self.patch(self.ui, "hide_error", self._set_called)
        email = 'valid@email'
        password = '123456'
        self.ui.ui.email_edit.setText(email)
        self.ui.ui.password_edit.setText(password)
        self.ui.login()
        self.assertEqual(self._overlay_show_counter, 1)
        self.assertFalse(self.ui.isEnabled())
        self.assert_backend_called('login_and_ping',
            self.app_name, email, password, self.ping_url)
        expected = ((), {})
        self.assertEqual(expected, self._called)

    def test_login_without_ping(self):
        """Test the login method."""
        self.patch(self.ui, "hide_error", self._set_called)
        self.ui.ping_url = ''
        email = 'valid@email'
        password = '123456'
        self.ui.ui.email_edit.setText(email)
        self.ui.ui.password_edit.setText(password)
        self.ui.login()
        self.assertEqual(self._overlay_show_counter, 1)
        self.assertFalse(self.ui.isEnabled())
        self.assert_backend_called('login',
            self.app_name, email, password)
        expected = ((), {})
        self.assertEqual(expected, self._called)

    def test_on_login_error(self):
        """Test the on_login_error method."""
        self.patch(self.ui, "show_error", self._set_called)
        app_name = 'my_app'
        self.ui.app_name = app_name
        error = {'errtype': 'UserNotValidated'}
        self.ui.on_login_error(app_name, error)
        self.assertEqual(self._overlay_hide_counter, 0)
        self.assertTrue(self.ui.isEnabled())
        expected = ((self.ui, 'my_app', ''), {})
        self.assertTrue(expected, self._called)

    def test_on_logged_in(self):
        """Test the on_login_in method."""
        app_name = 'my_app'
        email = 'email@example'
        self.ui.app_name = app_name
        self.ui.ui.email_edit.setText(email)

        def slot(app, email):
            """Fake slot."""
            self.signals_results.append((app, email))
        self.ui.userLoggedIn.connect(slot)
        self.ui.on_logged_in(app_name, None)
        self.assertEqual(self._overlay_hide_counter, 1)
        self.assertTrue(self.ui.isEnabled())
        self.assertIn((app_name, email), self.signals_results)

    def test_on_forgotten_password(self):
        """Test the on_forgotten_password method."""

        def slot():
            """Fake slot."""
            self.signals_results.append(1)
        self.ui.passwordForgotten.connect(slot)
        self.ui.on_forgotten_password()
        self.assertIn(1, self.signals_results)

    def test_on_forgotten_password_link_clicked(self):
        """Test the on_forgotten_password method."""

        def slot():
            """Fake slot."""
            self.signals_results.append(1)
        self.ui.passwordForgotten.connect(slot)
        self.ui.ui.forgot_password_label.linkActivated.emit("link")
        self.assertIn(1, self.signals_results)
