# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Setup Account page Qt UI."""

from PyQt4 import QtGui, QtCore

from ubuntu_sso.qt import forgotten_password_page
from ubuntu_sso.qt.tests import (
    BaseTestCasePage,
    FakePageUiStyle,
)

# is ok to access private method/attrs in tests
# pylint: disable=W0212, E1101


class ForgottenPasswordTestCase(BaseTestCasePage):
    """Test the SetupAccountPage code."""

    ui_class = forgotten_password_page.ForgottenPasswordPage
    ui_siganls = ('passwordResetTokenSent',)
    ui_backend_siganls = ('PasswordResetTokenSent', 'PasswordResetError')

    def test_request_new_password(self):
        """Test the request_new_password function."""
        app_name = 'my_app'
        email = 'email@example.com'
        self.ui.app_name = app_name
        self.ui.ui.email_line_edit.setText(email)
        self.ui.request_new_password()
        self.assertEqual(self._overlay_show_counter, 1)
        self.assertFalse(self.ui.isEnabled())
        self.assert_backend_called('request_password_reset_token',
            app_name, email)

    def test_email_widget(self):
        """Test the email_widget property."""
        value = self.ui.email_widget
        self.assertEqual(value, self.ui.ui.email_widget)
        self.assertTrue(isinstance(value, QtGui.QWidget))

    def test_forgotted_password_intro_label(self):
        """Test the forgotted_password_intro_label property."""
        value = self.ui.forgotted_password_intro_label
        self.assertEqual(value, self.ui.ui.forgotted_password_intro_label)
        self.assertTrue(isinstance(value, QtGui.QLabel))

    def test_email_address_label(self):
        """Test the email_address_label property."""
        value = self.ui.email_address_label
        self.assertEqual(value, self.ui.ui.email_address_label)
        self.assertTrue(isinstance(value, QtGui.QLabel))

    def test_email_address(self):
        """Test the email_address property."""
        value = self.ui.email_address
        self.assertEqual(value, str(self.ui.ui.email_line_edit.text()))
        self.assertTrue(isinstance(value, str))

    def test_email_address_line_edit(self):
        """Test the email_address_line_edit property."""
        value = self.ui.email_address_line_edit
        self.assertEqual(value, self.ui.ui.email_line_edit)
        self.assertTrue(isinstance(value, QtGui.QLineEdit))

    def test_send_button(self):
        """Test the send_button property."""
        value = self.ui.send_button
        self.assertEqual(value, self.ui.ui.send_button)
        self.assertTrue(isinstance(value, QtGui.QPushButton))

    def test_send_button_clicked(self):
        """Test the send_button property."""
        self.patch(self.ui, "hide_error", self._set_called)
        self.ui.send_button.clicked.emit(True)
        self.assert_backend_called('request_password_reset_token',
            self.app_name, '')
        expected = ((), {})
        self.assertEqual(expected, self._called)

    def test_try_again_widget(self):
        """Test the try_again_widget property."""
        value = self.ui.try_again_widget
        self.assertEqual(value, self.ui.ui.try_again_widget)
        self.assertTrue(isinstance(value, QtGui.QWidget))

    def test_try_again_button(self):
        """Test the try_again_button property."""
        value = self.ui.try_again_button
        self.assertEqual(value, self.ui.ui.try_again_button)
        self.assertTrue(isinstance(value, QtGui.QPushButton))

    def test_initialize_page_with_email(self):
        """Test the initialization method."""
        self.ui.ui.email_line_edit.setText('mail@example')
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "send_button", button)
        self.ui.initializePage()
        self.assertTrue(button.properties['default'])
        self.assertTrue(button.isEnabled())

    def test_set_translated_strings(self):
        """Test the translated string method."""
        app_name = 'my_app'
        self.ui.app_name = app_name
        self.ui._set_translated_strings()
        expected = (forgotten_password_page.REQUEST_PASSWORD_TOKEN_LABEL %
            {'app_name': app_name})
        self.assertEqual(self.ui.ui.forgotted_password_intro_label.text(),
            expected)
        self.assertEqual(self.ui.ui.email_address_label.text(),
            forgotten_password_page.EMAIL_LABEL)
        self.assertEqual(self.ui.ui.send_button.text(),
            forgotten_password_page.RESET_PASSWORD)
        self.assertEqual(self.ui.ui.try_again_button.text(),
            forgotten_password_page.TRY_AGAIN_BUTTON)

    def test_connect_ui(self):
        """Test the connect ui method."""
        self.ui._connect_ui()
        # We expect 2 values because _connect_ui is called in the init too.
        # For email_line_edit we expect 3 because has another
        # connection in the .ui file.
        self.assertEqual(self.ui.ui.email_line_edit.receivers(
            QtCore.SIGNAL("textChanged(const QString&)")), 3)
        self.assertEqual(self.ui.ui.send_button.receivers(
            QtCore.SIGNAL("clicked()")), 2)
        self.assertEqual(self.ui.ui.try_again_button.receivers(
            QtCore.SIGNAL("clicked()")), 2)

    def test_validate_not_valid(self):
        """Test validate method."""
        self.ui.email_address_line_edit.setText('')
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "send_button", button)
        self.ui._validate()
        self.assertFalse(button.isEnabled())

    def test_validate_valid(self):
        """Test validate method."""
        self.ui.email_address_line_edit.setText('mail@example')
        button = FakePageUiStyle()
        self.patch(self.ui.ui, "send_button", button)
        self.ui._validate()
        self.assertTrue(button.isEnabled())

    def test_on_try_again(self):
        """Test on_try_again method."""
        self.patch(self.ui, "hide_error", self._set_called)
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.on_try_again()
        self.assertTrue(self.ui.email_widget.isVisible())
        self.assertFalse(self.ui.try_again_widget.isVisible())
        expected = ((), {})
        self.assertEqual(expected, self._called)

    def test_on_password_reset_token_sent(self):
        """Test on_password_reset_token_sent method."""

        def slot():
            """Fake slot."""
            self.signals_results.append(1)
        self.ui.passwordResetTokenSent.connect(slot)
        self.ui.on_password_reset_token_sent('app_name', {})
        self.assertEqual(self._overlay_hide_counter, 1)
        self.assertTrue(self.ui.isEnabled())
        self.assertTrue(1 in self.signals_results)

    def test_on_password_reset_error(self):
        """Test on_password_reset_error method."""
        self.patch(self.ui, "show_error", self._set_called)
        app_name = 'my_app'
        error = {'errtype': 'AnotherError'}
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.on_password_reset_error(app_name, error)
        expected = ((self.ui, app_name,
            forgotten_password_page.REQUEST_PASSWORD_TOKEN_TECH_ERROR), {})
        self.assertTrue(expected, self._called)
        self.assertFalse(self.ui.email_widget.isVisible())
        self.assertFalse(self.ui.forgotted_password_intro_label.isVisible())
        self.assertTrue(self.ui.try_again_widget.isVisible())
        self.assertEqual(self._overlay_hide_counter, 1)
        self.assertTrue(self.ui.isEnabled())

    def test_on_password_reset_error_with_token_error(self):
        """Test on_password_reset_error method."""
        self.patch(self.ui, "show_error", self._set_called)
        app_name = 'my_app'
        error = {'errtype': 'ResetPasswordTokenError'}
        self.ui.on_password_reset_error(app_name, error)
        expected = ((self.ui, app_name,
            forgotten_password_page.REQUEST_PASSWORD_TOKEN_WRONG_EMAIL), {})
        self.assertTrue(expected, self._called)
